@@ -23,7 +23,8 @@ namespace llvm {
   class LLVMCConfigurationEmitter : public TableGenBackend {
     RecordKeeper &Records;
   public:
-    explicit LLVMCConfigurationEmitter(RecordKeeper &R) : Records(R) {}
+    explicit LLVMCConfigurationEmitter(RecordKeeper &records) : 
+      Records(records) {}
 
     // run - Output the asmwriter, returning true on failure.
     void run(raw_ostream &o);
